%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Example of Matlab program using QPALM as an optimization solver.
%  QPALM can solve a minimization problem of the form
%
%      minimize (in x)  g'*x + 0.5*x'*H*x
%      subject to       lbx <=    x <= ubx
%                       lbi <= AI*x <= ubi
%                       AE*x = be
%
%  The program RANDQP is executed by typing
%
%      randqp
%
%  in the Matlab window.
%
%  Author: J.Ch. Gilbert, INRIA, 2013
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------
% Initialization
%---------------

  clf;
  clear all;
  format compact
  format long

%--------------------------------------
% Choose the file of the example to run
%--------------------------------------

 datafile = 'randqp10a.tdt';		% unbounded problem (nullspace // e1+e2), no constraint
% datafile = 'randqp10b.tdt';		% unbounded problem (nullspace // e1+e2), bound constraints
% datafile = 'randqp10c.tdt';		% (nullspace // e1+e2), bound constraints, one solution
% datafile = 'randqp10d.tdt';		% equality constraints, with a solution
% datafile = 'randqp10e.tdt';		% equality constraints, unbounded if feasible
% datafile = 'randqp10f.tdt';		% equality constraints, infeasible problem
% datafile = 'randqp10g.tdt';		% equality constraints and bounds on x, unbounded problem
% datafile = 'randqp10h.tdt';		% equality constraints and bounds on x, with a solution
% datafile = 'randqp10i.tdt';		% equality, inequality constraints, bounds on x, with a solution
% datafile = 'randqp10j.tdt';		% equality, inequality constraints, bounds on x, unbounded if feasible
% datafile = 'randqp10k.tdt';		% equality, inequality constraints, bounds on x, infeasible

% datafile = 'randqp100a.tdt';		% no constraint with a solution
% datafile = 'randqp100b.tdt';		% bound constraints with a solution
% datafile = 'randqp100c.tdt';		% (nullspace // e1+e2), bound constraints, one solution

% What follows is no longer valid ... (2013-03-22)
% datafile = 'randqp1.dat';
% datafile = 'randqp2.dat';		% singular matrix
% datafile = 'randqp1000.dat';
% datafile = 'data.dat';

%------------------------
% General initializations
%------------------------

  addpath '../src'			% Add the QPALM directory to the Matlab path

%--------------------------------------------
% Some paramaters and choice of the QP solver
%--------------------------------------------

% fid = fopen('res','w');
  fid = 1;

%--------------
% Read the data
%--------------

  fprintf(fid,'\n*** Problem ''%s'' ***\n',datafile);

  % definition du POQ
  %
  %   min   g'*x + 0.5*x'*H*x
  %   s.t.  lbx <=    x <= ubx
  %         lbi <= AI*x <= ubi
  %         AE*x = be

  % Read data on the 'datafile' (the format of the datafile is revealed by the way it is read below)

  fdat = fopen(datafile,'r');
  if fdat < 0
    fprintf ('\n### Cannot open file ''%s''\n\n',datafile);
    return
  end
  n = sscanf(fgets(fdat),'%i',1);		% nb of variables
  g = fscanf(fdat,'%e',n);			% gradient
  fgets(fdat);					% next line
  nnz = sscanf(fgets(fdat),'%i',1);		% type of Hessian
  if nnz == 0					% H is full
    H = fscanf(fdat,'%e',[n,n]);
  elseif nnz > 0				% H is sparse
    for k = 1:nnz
      i(k) = fscanf(fdat,'%i',1);
      j(k) = fscanf(fdat,'%i',1);
      v(k) = fscanf(fdat,'%e',1);
    end
    H = sparse(i,j,v,n,n);
  else
    fprintf ('\n### nnz (=%i) for H should be >= 0\n\n',nnz);
    return
  end
  fgets(fdat);					% next line

  % bound constraints on x

  lbx = -inf*ones(n,1);
  nlx = sscanf(fgets(fdat),'%i',1);		% nb of lower bounds on x
  if nlx
    for i=1:nlx
      j = fscanf(fdat,'%i',1);
      lbx(j) = fscanf(fdat,'%e',1);
    end
    fgets(fdat);				% next line
  end
  ubx = inf*ones(n,1);
  nux = sscanf(fgets(fdat),'%i',1);		% nb of upper bounds on x
  if nux
    for i=1:nux
      j = fscanf(fdat,'%i',1);
      ubx(j) = fscanf(fdat,'%e',1);
    end
    fgets(fdat);				% next line
  end

  % Inequality constraints

  mi = sscanf(fgets(fdat),'%i',1);

  if mi

    nnz = sscanf(fgets(fdat),'%i',1);		% type of Jacobian
    if nnz == 0					% AI is full
      AI = fscanf(fdat,'%e',[mi,n]);
    elseif nnz > 0				% AI is sparse
      for k = 1:nnz
        i(k) = fscanf(fdat,'%i',1);
        j(k) = fscanf(fdat,'%i',1);
        v(k) = fscanf(fdat,'%e',1);
      end
      AI = sparse(i,j,v,mi,n);
    else
      fprintf ('\n### nnz (=%i) for AI should be >= 0\n\n',nnz);
      return
    end
    fgets(fdat);				% next line

    lbi = -inf*ones(mi,1);
    nli = sscanf(fgets(fdat),'%i',1);		% nb of lower inequality constraints
    if nli
      for i=1:nli
        j = fscanf(fdat,'%i',1);
        lbi(j) = fscanf(fdat,'%e',1);
      end
      fgets(fdat);				% next line
    end

    ubi = inf*ones(mi,1);
    nui = sscanf(fgets(fdat),'%i',1);		% nb of upper inequality constraints
    if nui
      for i=1:nui
        j = fscanf(fdat,'%i',1);
        ubi(j) = fscanf(fdat,'%e',1);
      end
      fgets(fdat);				% next line
    end

  else

    AI = [];
    lbi = [];
    ubi = [];

  end

  % Equality constraints

  me = sscanf(fgets(fdat),'%i',1);
  if me
    nnz = sscanf(fgets(fdat),'%i',1);		% type of Jacobian
    if nnz == 0					% AE is full
      AE = fscanf(fdat,'%e',[me,n]);
    elseif nnz > 0				% AE is sparse
      for k = 1:nnz
        i(k) = fscanf(fdat,'%i',1);
        j(k) = fscanf(fdat,'%i',1);
        v(k) = fscanf(fdat,'%e',1);
      end
      AE = sparse(i,j,v,me,n);
    else
      fprintf ('\n### nnz (=%i) for AE should be >= 0\n\n',nnz);
      return
    end
    be = fscanf(fdat,'%e',me);
  else
    AE = [];
    be = [];
  end
  fclose(fdat);

  % Undefned data


  % Print data

% x = zeros(n,1);
% fprintf('\n i      lbi(i)         x(i)         ubi(i)');
% for i=1:n
%   fprintf('\n%2i  %12.5e  %12.5e  %12.5e',i,lbx(i),x(i),ubx(i));
%   if x(i) <= lbx(i)+1.e-8; fprintf('  bound-'); end
%   if x(i) >= ubx(i)-1.e-8; fprintf('  bound+'); end
% end

  eigH = eig(H);
  eigHmin = min(eigH);
  eigHmax = max(eigH);
  fprintf(fid,'\n\nEigenvalues of H: min = %9.2e, max = %8.2e, cond = %8.2e\n',eigHmin,eigHmax,eigHmax/eigHmin);

%>  % Direct solver
%>
%>  fprintf('\nComputing by a direct solver\n');
%>
%>  time_init = clock;
%>  x0 = -H\g;
%>  fprintf(fid,'elapsed_time in a direct solve = %g sec',etime(clock,time_init));
%>
%>  % Direct solver
%>
%>  fprintf('\n\nComputing by a Cholesky factorization\n');
%>
%>  time_init = clock;
%>  [R,p] = chol(H);
%>  if p	% H is not positive definite
%>    return
%>  end
%>  x0 = -(R')\g;
%>  x0 = R\x0;
%>  fprintf(fid,'elapsed_time in a Cholesky factorization = %g sec',etime(clock,time_init));

%d=zeros(n,1);
%d(1)=1;
%d(2)=1;
%d=-d;
%AId=AI*d
%lbi
%ubi
%AEd=AE*d
%stop

%----------------------
% Solve the QP by QPALM
%----------------------

  fprintf(fid,'\nRunning QPALM\n');

  x0 = [];

  options.feas  = 1.e-10;
  options.feass = 1.e-10;
% options.dxmin = 1.e-20;
  options.gtol  = 1.e-12;
  options.cgtol = 1.e-12;
  options.fout  = fid;
  options.verb  = 4;
% options.rctl  = 'fixed';
  options.rlag  = 1.0e+0;
% options.dcr   = 1.e-5;
  options.alit  = 20;
% options.cgit  = 50;
  options.lslv  = 'cg';
% options.lslv  = 'chol';

  time_init = clock;
  [x1,lm1,info] = qpalm(g,H,lbx,ubx,AI,lbi,ubi,AE,be,x0,[],options);
  fprintf(fid,'elapsed_time in QPALM = %g sec\n',etime(clock,time_init));

  switch info.flag
  case 0
  case 1
    fprintf(fid,'\nsmallest feasible shift s');
    fprintf(fid,'\n  %12.5e',info.s);
  case 2
    return
  case 7
    fprintf(fid,'\nx =');
    fprintf(fid,'\n  %12.5e',x1);
    fprintf(fid,'\ndirection of unboundedness d');
    fprintf(fid,'\n  %12.5e',info.ubdd);
    fprintf(fid,'\nH*d =');
    fprintf(fid,'\n  %12.5e',H*info.ubdd);
% otherwise
%   return
  end

  %-------------------------
  % solve the QP by QUADPROG
  %-------------------------

  fprintf(fid,'\n\nRunning QUADPROG\n');

  qp_options = optimset( ...
       'LargeScale','off', ...
       'Display','iter', ...
       'TolFun',1e-8, ...
       'TolX',1e-10);

  tic;
  if mi
    [x2,f,exitflag] = quadprog(H,g,[AI;-AI],[ubi;-lbi],AE,be,lbx,ubx,x0,qp_options);
  else
    [x2,f,exitflag] = quadprog(H,g,[],[],AE,be,lbx,ubx,x0,qp_options);
  end
  fprintf(fid,'elapsed_time in QUADPROG = %g sec',toc);

  % Check optimality at the current x. It is checked whether the
  % projected gradient 'pgr' is small enough. Recall that the projected
  % gradient 'pgr' is defined by
  %
  %             { min(0,gr(i))  if lbx(i) = x(i)
  %    pgr(i) = { gr(i)         if lbx(i) < x(i) < ubx(i)
  %             { max(0,gr(i))  if          x(i) = ubx(i),
  %
  %  where gr is the gradient of the objective.

  pgr = g+H*x2;
  Il = find(x2<=lbx+options.feas);	% indices of the variables at the lower bound
  pgr(Il) = min(0,pgr(Il));
  Iu = find(ubx-options.feas<=x2);	% indices of the variables at the upper bound
  pgr(Iu) = max(0,pgr(Iu));

  fprintf(fid,'\nActive bounds:\n. %i lower bounds',length(Il));
  if length(Il) > 0
    fprintf(fid,':');
    fprintf(fid,' %i',Il);
  end
  fprintf(fid,'\n. %i upper bounds',length(Iu));
  if length(Iu) > 0
    fprintf(fid,':');
    fprintf(fid,' %i',Iu);
  end

  gopt = norm(pgr,inf);		% test convergence
  fprintf(fid,'\n|Projected gradient| = %12.5e',gopt);

  %--------------------------
  % Compare the two solutions
  %--------------------------

  fprintf(fid,'\n\nDifference between the two algorithms');
  x1n = norm(x1);
  if x1n ~= 0
    fprintf(fid,'\n- primal (relative): %12.5e\n',norm(x2-x1)/x1n);
  else
    fprintf(fid,'\n- primal (absolute): %12.5e\n',norm(x2-x1));
  end

  fprintf(fid,'\nindex       lbx        x (qpalm)   x (quadprog)       ubx');
  dmax = 0;
  for i=1:n
    fprintf(fid,'\n%5i  %12.5e  %12.5e  %12.5e  %12.5e',i,lbx(i),x1(i),x2(i),ubx(i));
    if abs(x2(i)-x1(i)) > dmax
      imax = i;
      dmax = abs(x2(i)-x1(i));
    end
  end
  fprintf(fid,'\nMax diff %12.5e, at index %i\n',dmax,imax);

  return
