%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Generator of random QP data.
%
%  The data are generated in the files xxx.tdt (ascii file) and xxx.bdt
%  (binary file). For the format of the files, see the comment in the
%  generated file xxx.tdt and see how they are written below and in the
%  file 'randqp.m'. The format allows for dense and sparse data.
%
%  Author: J.Ch. Gilbert, INRIA, December 2013
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------
% Initialization
%---------------

  clear all;

%---------------
% Initialization
%---------------

  job = 19;	%  1: unbounded problem (nullspace // e1+e2), no constraint
                %  2: unbounded problem (nullspace // e1+e2), bound constraints
		%  3: (nullspace // e1+e2), bound constraints, one solution
		%  4: equality constraints, with a solution
		%  5: equality constraints, unbounded if feasible
		%  6: equality constraints, infeasible problem
		%  7: equality constraints and bounds on x, unbounded problem
		%  8: equality constraints and bounds on x, with a solution
		%  9: equality, inequality+equality constraints, bounds on x, with a solution
		% 10: equality, inequality+equality constraints, bounds on x, unbounded if feasible
		% 11: equality, inequality+equality constraints, bounds on x, infeasible
		% 12: equality, inequality+equality constraints, with a solution
                % 13: no constraint, nullspace = vect{e5,e7}, unbounded along e5 and e7
                % 14: no constraint, nullspace = vect{e5,e7}, unbounded along e7
                % 15: no constraint, nullspace = vect{e5,e7}, with solution
                % 16: bound constraint on x5, nullspace = vect{e5,e7}, unbounded along e7
                % 17: inequality constraint on x4+x5, nullspace = vect{e5,e7}, unbounded along e7
		% 18: nonconvex unconstrained problem
		% 19: bound constrained, unboudedness detectable by the first GP step

%---------------
% Open datafiles
%---------------

  fbin = fopen('xxx.bdt','w');
  if fbin < 0
    fprintf ('\n### Cannot open file ''%s'' for writing\n\n','xxx.bdt');
    return
  end
  ftxt = fopen('xxx.tdt','w');
  if ftxt < 0
    fprintf ('\n### Cannot open file ''%s'' for writing\n\n','xxx.tdt');
    return
  end

%------------------------
% General initializations
%------------------------

  reset(RandStream.getDefaultStream);	% reset the default random stream to its initial state

%---------------------------------------------
% Generate the data and write them on the file
%---------------------------------------------

  % dimension
  % ^^^^^^^^^

  n = 10;

  fwrite(fbin,n,'int32');	% 'integer*4' in C or Fortran, integer on 32 bits
  fprintf(ftxt,'%0i          number n of variables, followed by the gradient at the origin g\n',n);

  % Gradient at the origin
  % ^^^^^^^^^^^^^^^^^^^^^^

  g = rand(n,1);
  if job == 14
    g(5) = 0;		% e5 is a flat zero curvature direction
  elseif job == 15
    g(5) = 0;		% e5 is a flat zero curvature direction
    g(7) = 0;		% e7 is a flat zero curvature direction
  elseif job == 19
    g = 2*g-1;
  end

  fwrite(fbin,g,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
  fprintf(ftxt,'%23.16e\n',g);

  % Hessian
  % ^^^^^^^

  H = rand(n,n); L = tril(H);
  if job <= 12
    L(2,:) = -L(1,:);	% impose that e1+e2 is in the null space of H
  elseif (job >= 13) & (job <= 17)
    L(5,:) = 0;		% impose that e5 is in the null space of H
    L(7,:) = 0;		% impose that e7 is in the null space of H
  elseif (job == 18) | (job == 19)
    L(5,:) = 0;		% impose that e5 is in the null space of H
  end
  H=L*L';
  if job == 18
    H(5,5) = -1;
  end
  % Control the conditioning of H
  eigH = eig(H);
  eigHmin = min(eigH);
  eigHmax = max(eigH);
  fprintf('\nEigenvalues of H: min = %9.2e, max = %8.2e, cond = %8.2e\n',eigHmin,eigHmax,eigHmax/eigHmin);
%   while eigHmin < 0
%     L = L-eigHmin*eye(n);
%     L(2,:) = -L(1,:); % impose that e1+e2 is in the null space of H
%     H=L*L';
%     eigH = eig(H);
%     eigHmin = min(eigH);
%     eigHmax = max(eigH);
%     fprintf('Eigenvalues of H: min = %9.2e, max = %8.2e\n',eigHmin,eigHmax);
%   end

  % Write on files

  fwrite(fbin,0,'int32');	% H is a dense matrix
  fprintf(ftxt,'%0i          the Hessian H is a dense matrix, with the following elements\n',0);

  fwrite(fbin,H,'double');
  fprintf(ftxt,'%23.16e\n',H);

  % Bounds on x

  %----------
  if (job == 1) | ((job >= 13) & (job <= 15)) | (job == 18)
  %----------

    % Bound constraints

    nlbx = 0;
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nlbx);

    nubx = 0;
    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i          number of upper bounds on x, followed by their indices and values\n',nubx);

    % Inequality constraints

    mi = 0;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i          number mi of inequality constraints\n',mi);

    % Equality constraints

    me = 0;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i          number me of equality constraints\n',me);

  %--------------
  elseif job == 16
  %--------------

    % Bound constraints

    nlbx = 1;
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nlbx);

    lbx(5) = -1;
    fwrite(fbin,5,'int32');
    fwrite(fbin,lbx(5),'double');
    fprintf(ftxt,'%0i  %23.16e\n',5,lbx(5));

    nubx = 1;
    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i          number of upper bounds on x, followed by their indices and values\n',nubx);

    ubx(5) = 1;
    fwrite(fbin,5,'int32');
    fwrite(fbin,ubx(5),'double');
    fprintf(ftxt,'%0i  %23.16e\n',5,ubx(5));

    % Inequality constraints

    mi = 0;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i          number mi of inequality constraints\n',mi);

    % Equality constraints

    me = 0;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i          number me of equality constraints\n',me);

  %--------------
  elseif job == 17
  %--------------

    % Bound constraints

    nlbx = 0;
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nlbx);

    nubx = 0;
    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i          number of upper bounds on x, followed by their indices and values\n',nubx);

    % Inequality constraints

    mi = 1;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i          number mi of inequality constraints\n',mi);

    fwrite(fbin,0,'int32');	% AI is a dense matrix
    fprintf(ftxt,'%0i          the Jacobian AI of the inequality constraints is a dense matrix, with the following elements\n',0);

    AI = zeros(1,n);
    AI(1,4) = 1;
    AI(1,5) = 1;
    fwrite(fbin,AI,'double');
    fprintf(ftxt,'%23.16e\n',AI);

    lbi = -2;
    fwrite(fbin,1,'int32');
    fprintf(ftxt,'%0i          number of lower inequality constraints, followed by their indices and values\n',1);
    fwrite(fbin,1,'int32');
    fwrite(fbin,lbi,'double');
    fprintf(ftxt,'%0i  %23.16e\n',1,lbi);

    ubi =  2;
    fwrite(fbin,1,'int32');
    fprintf(ftxt,'%0i          number of upper inequality constraints, followed by their indices and values\n',1);
    fwrite(fbin,1,'int32');
    fwrite(fbin,ubi,'double');
    fprintf(ftxt,'%0i  %23.16e\n',1,ubi);

    % Equality constraints

    me = 0;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i          number me of equality constraints\n',me);

  %--------------
  elseif job == 2
  %--------------

    % bound constraints

    nlbx = min(6,n-2);
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i\n',nlbx);

    for i = 3:3+nlbx-1

      lbx(i) = -10*rand(1);

      fwrite(fbin,i,'int32');
      fwrite(fbin,lbx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,lbx(i));

    end

    nubx = nlbx;

    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i\n',nubx);

    for i = n-nubx+1:n

      ubx(i) = 10*rand(1);

      fwrite(fbin,i,'int32');
      fwrite(fbin,ubx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,ubx(i));

    end

    % Inequality constraints

    mi = 0;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i\n',mi);

    % Equality constraints

    me = 0;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i\n',me);

  %--------------
  elseif job == 3
  %--------------

    nlbx = min(6,n-1);
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i\n',nlbx);

    for i = 2:2+nlbx-1

      lbx(i) = -10*rand(1);

      fwrite(fbin,i,'int32');
      fwrite(fbin,lbx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,lbx(i));

    end

    nubx = nlbx;

    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i\n',nubx);

    for i = n-nubx+1:n

      ubx(i) = 10*rand(1);

      fwrite(fbin,i,'int32');
      fwrite(fbin,ubx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,ubx(i));

    end

    % Inequality constraints

    mi = 0;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i\n',mi);

    % Equality constraints

    me = 0;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i\n',me);

  %-----------------------------
  elseif (job >= 4) & (job <= 6)
  %-----------------------------

    nlbx = 0;
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i\n',nlbx);

    nubx = 0;
    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i\n',nubx);

    % Inequality constraints

    mi = 0;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i\n',mi);

    % Equality constraints

    me = 5;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i\n',me);

    fwrite(fbin,0,'int32');	% AE is a dense matrix
    fprintf(ftxt,'%0i\n',0);

    AE = rand(me,n)*2-1;
    if job == 5			% impose that e1+e2 is in the null space of AE
      AE(:,2) = - AE(:,1);
    end
    if job == 6			% make the equality constraints infeasible
      AE(1,:) = zeros(1,n);
      be(1) = 1;
    end
    fwrite(fbin,AE,'double');
    fprintf(ftxt,'%23.16e\n',AE);

    be = rand(me,1)*2-1;
    fwrite(fbin,be,'double');
    fprintf(ftxt,'%23.16e\n',be);

  %-----------------------------
  elseif (job == 7) | (job == 8)
  %-----------------------------

    % bound constraints

    nlbx = min(6,n-2);
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i\n',nlbx);

    for i = 3:3+nlbx-1

      lbx(i) = -10*rand(1);

      fwrite(fbin,i,'int32');
      fwrite(fbin,lbx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,lbx(i));

    end

    nubx = nlbx;

    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i\n',nubx);

    for i = n-nubx+1:n

      ubx(i) = 10*rand(1);

      fwrite(fbin,i,'int32');
      fwrite(fbin,ubx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,ubx(i));

    end

    % Inequality constraints

    mi = 0;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i\n',mi);

    % Equality constraints

    me = 5;
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i\n',me);

    fwrite(fbin,0,'int32');	% AE is a dense matrix
    fprintf(ftxt,'%0i\n',0);

    AE = rand(me,n)*2-1;
    if job == 7			% impose that e1+e2 is in the null space of AE
      AE(:,2) = - AE(:,1);
    end
    fwrite(fbin,AE,'double');
    fprintf(ftxt,'%23.16e\n',AE);

    be = rand(me,1)*2-1;
    fwrite(fbin,be,'double');
    fprintf(ftxt,'%23.16e\n',be);

  %------------------------------
  elseif (job >= 9) & (job <= 12)
  %------------------------------

    % bound constraints

    if job >= 12

      nlbx = 0;
      fwrite(fbin,nlbx,'int32');
      fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nlbx);

      nubx = 0;
      fwrite(fbin,nubx,'int32');
      fprintf(ftxt,'%0i          number of upper bounds on x, followed by their indices and values\n',nubx);

    else

      nlbx = min(6,n-2);
      fwrite(fbin,nlbx,'int32');
      fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nlbx);

      for i = 3:3+nlbx-1		% no bounds on e1+e2
        lbx(i) = -10*rand(1);
        fwrite(fbin,i,'int32');
        fwrite(fbin,lbx(i),'double');
        fprintf(ftxt,'%0i  %23.16e\n',i,lbx(i));
      end

      nubx = nlbx;
      fwrite(fbin,nubx,'int32');
      fprintf(ftxt,'%0i          number of upper bounds on x, followed by their indices and values\n',nubx);

      for i = n-nubx+1:n		% no bounds on e1+e2
        ubx(i) = 10*rand(1);
        fwrite(fbin,i,'int32');
        fwrite(fbin,ubx(i),'double');
        fprintf(ftxt,'%0i  %23.16e\n',i,ubx(i));
      end

    end

    % Inequality constraints

    mi = 5;
    fwrite(fbin,mi,'int32');
    fprintf(ftxt,'%0i          number mi of inequality constraints\n',mi);

    fwrite(fbin,0,'int32');	% AI is a dense matrix
    fprintf(ftxt,'%0i          the Jacobian AI of the inequality constraints is a dense matrix, with the following elements\n',0);

    AI = rand(mi,n)*2-1;
    fwrite(fbin,AI,'double');
    fprintf(ftxt,'%23.16e\n',AI);

    lbi = -10*rand(mi,1);
    ubi =  10*rand(mi,1);
    s = -(g(1)+g(2));		% the unboundedness direction will be s*(e1+e2)
    if job == 10
      for i = 1:mi			% impose that s*R_+*(e1+e2) is feasible for the inequalities
        r = s*(AI(i,1)+AI(i,2));
        if r < 0
          lbi(i) = -inf;
        elseif r > 0
          ubi(i) =  inf;
        end
      end
    end

    finite_lbi = find(lbi > -inf);
    nlbi = length(finite_lbi);
    fwrite(fbin,nlbi,'int32');
    fprintf(ftxt,'%0i          number of lower inequality constraints, followed by their indices and values\n',nlbi);
    for i = finite_lbi'
      fwrite(fbin,i,'int32');
      fwrite(fbin,lbi(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,lbi(i));
    end

    finite_ubi = find(ubi < inf);
    nubi = length(finite_ubi);
    fwrite(fbin,nubi,'int32');
    fprintf(ftxt,'%0i          number of upper inequality constraints, followed by their indices and values\n',nubi);
    for i = finite_ubi'
      fwrite(fbin,i,'int32');
      fwrite(fbin,ubi(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,ubi(i));
    end

    % Equality constraints

    me = 5;
    if (job == 11); me = 10; end
    fwrite(fbin,me,'int32');
    fprintf(ftxt,'%0i          number me of equality constraints\n',me);

    fwrite(fbin,0,'int32');	% AE is a dense matrix
    fprintf(ftxt,'%0i          the Jacobian AE of the equality constraints is a dense matrix, with the following elements\n',0);

    AE = rand(me,n)*2-1;
    if job == 10		% impose that e1+e2 is in the null space of AE
      AE(:,2) = - AE(:,1);
    end
    fwrite(fbin,AE,'double');
    fprintf(ftxt,'%23.16e\n',AE);

    be = rand(me,1)*2-1;
    fwrite(fbin,be,'double');
    fprintf(ftxt,'%23.16e\n',be);

  %------------------------------
  elseif job >= 19
  %------------------------------

    % bound constraints

    g

    ilbx = setdiff(find(g>0)',5)
    nlbx = length(ilbx);
    fwrite(fbin,nlbx,'int32');
    fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nlbx);

    for i = ilbx
      lbx(i) = -10*rand(1);
      fwrite(fbin,i,'int32');
      fwrite(fbin,lbx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,lbx(i));
    end

    iubx = setdiff(find(g<0)',5)
    nubx = length(iubx);
    fwrite(fbin,nubx,'int32');
    fprintf(ftxt,'%0i          number of lower bounds on x, followed by their indices and values\n',nubx);

    for i = iubx
      ubx(i) = 10*rand(1);
      fwrite(fbin,i,'int32');
      fwrite(fbin,ubx(i),'double');
      fprintf(ftxt,'%0i  %23.16e\n',i,ubx(i));
    end

  end

%----------------
% Close the files
%----------------

  fclose(fbin);
  fclose(ftxt);

  return
