%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Example of Matlab program using QPALM as an optimization solver.
%  QPALM can solve a minimization problem of the form
%
%      minimize (in x)  g'*x + 0.5*x'*H*x
%      subject to       lbx <=    x <= ubx
%                       lbi <= AI*x <= ubi
%                       AE*x = be
%
%  The program RANDQP is executed by typing
%
%      randqp
%
%  in the Matlab window.
%
%  Author: J.Ch. Gilbert, INRIA, 2014
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------
% Initialization
%---------------

  clf;
  clear all;
  format compact
  format long

  start_with_good_x   = false;	% set to true if the primal solution has to be taken on entry of Qpalm
  start_with_good_lm  = false;	% set to true if the dual solution has to be taken on entry of Qpalm
  start_with_pert_lm  = false;	% set to true if the dual solution has to be taken on entry of Qpalm
  force_full_matrices = false;	% set to true to impose the data to be full, not sparse (to offer a fair comparison with Quadprog-active-set)
  pertubation_good_lm = 0.01;	% in case start_with_good_lm == true, give the percentage of perturbation of lmopt

%--------------------------------------
% Choose the file of the example to run
%--------------------------------------

% datafile = 'solved2a.tdt';		%+0 2 bounds, solved at the inital point 0
% datafile = 'solved2b.tdt';		%+1 2 inequality constraints, solved at the initial point 0 (must compute multiplier)

% datafile = 'special2a.tdt';		%+0 with bound, for which MT does not find an unboundedness direction
% datafile = 'special3a.tdt';		%+0 used to test some part of Qpalm, failure of Quadprog
% datafile = 'special3b.tdt';		%+1 used to test some part of Qpalm, linear problem
% datafile = 'special3c.tdt';		%+0 used in the COPI'14 conference and the MPC paper, failure of Quadprog
% datafile = 'special4a.tdt';		%+0 another failure of Quadprog

% datafile = 'randqp10a.tdt';		%+0 unbounded problem (nullspace // e1+e2), no constraint
% datafile = 'randqp10b.tdt';		%+0 unbounded problem (nullspace // e1+e2), bound constraints
% datafile = 'randqp10c.tdt';		%+0 (nullspace // e1+e2), bound constraints, one solution
% datafile = 'randqp10d.tdt';		%+1 equality constraints, with a solution
% datafile = 'randqp10e.tdt';		%+1 equality constraints, unbounded if feasible
% datafile = 'randqp10f.tdt';		%+3(car-teste-les--contraintes-translatees-apres-3it) equality constraints, infeasible problem
% datafile = 'randqp10g.tdt';		%+1 equality constraints and bounds on x, unbounded problem
% datafile = 'randqp10h.tdt';		%+1 equality constraints and bounds on x, with a solution
  datafile = 'randqp10i.tdt';		%+1 equality, inequality+equality constraints, bounds on x, with a solution
% datafile = 'randqp10j.tdt';		% equality, inequality+equality constraints, bounds on x, unbounded if feasible
% datafile = 'randqp10k.tdt';		%+3(car-teste-les--contraintes-translatees-apres-3it) equality, inequality+equality constraints, bounds on x, infeasible
% datafile = 'randqp10l.tdt';		%+1 equality, inequality+equality constraints, with a solution
% datafile = 'randqp10m.tdt';		% no constraint, nullspace = vect{e5,e7}, unbounded along e5 and e7
% datafile = 'randqp10n.tdt';		% no constraint, nullspace = vect{e5,e7}, unbounded along e7
% datafile = 'randqp10o.tdt';		% no constraint, nullspace = vect{e5,e7}, with solution
% datafile = 'randqp10p.tdt';		% bound constraint on x5, nullspace = vect{e5,e7}, unbounded along e7
% datafile = 'randqp10q.tdt';		% inequality constraint on x4+x5, nullspace = vect{e5,e7}, unbounded along e7
% datafile = 'randqp10r.tdt';		% nonconvex unconstrained problem

% datafile = 'cutest_aug3d.tdt';	% convergence 2 itérations de LA lorsque l'on part proche avec une annulation rapide et bizarre des contraintes
% datafile = 'cutest_primalc1.tdt';	%+1
% datafile = 'cutest_qpcboe12.tdt';	%+3 qpalm bien moins performant que qpb et même que Quadprog
% datafile = 'cutest_qscagr7.tdt';	%+3 qpalm bien moins performant que quadprog
% datafile = 'cutest_qscfxm1.tdt';	%+1 Gros calculs avec Qpalm (cpu disisé par 2 en creux). check5b and check1 done; error in qpalm_cholp; 0 looks optimal; cannot get feasibility
% datafile = 'cutest_values.tdt';	%+1

% datafile = 'randqp100a.tdt';		% no constraint with a solution
% datafile = 'randqp100b.tdt';		% bound constraints with a solution
% datafile = 'randqp100c.tdt';		% (nullspace // e1+e2), bound constraints, one solution

% datafile = 'gilbert-delbos-2005-ex43-0.tdt';		% 2 bounds

% datafile = 'cutest_qscagr7.tdt';	% TODO: pourquoi n'y a t-il pas d'identification finie de contraintes actives (la dernière iteration n'est pas du type GP+CG+GP)

% What follows is no longer valid ... (2013-03-22)
% datafile = 'randqp1.dat';
% datafile = 'randqp2.dat';		% singular matrix
% datafile = 'randqp1000.dat';
% datafile = 'data.dat';

%------------------------
% General initializations
%------------------------

  addpath '../src'			% Add the QPALM directory to the Matlab path

%--------------------------------------------
% Some paramaters and choice of the QP solver
%--------------------------------------------

% fid = fopen('res','w');
  fid = 1;

%--------------
% Read the data
%--------------

  fprintf(fid,'\n*** Problem ''%s'' ***\n\nDate: %s\n',datafile,datestr(now));

  % definition du POQ
  %
  %   min   g'*x + 0.5*x'*H*x
  %   s.t.  lbx <=    x <= ubx
  %         lbi <= AI*x <= ubi
  %         AE*x = be

  % Read data on the 'datafile' (the format of the datafile is revealed by the way it is read below)

  fdat = fopen(datafile,'r');
  if fdat < 0
    fprintf ('\n### Cannot open file ''%s''\n\n',datafile);
    return
  end

  % nb of variables

  n = fscanf(fdat,'%i',1);
  fgets(fdat);					% next line to skip the comment

  % gradient at the origin

  nnz = fscanf(fdat,'%i',1);			% type of gradient
  fgets(fdat);					% next line to skip the comment
  if nnz < 0					% g is full
    g = fscanf(fdat,'%e',n);
  else						% g is sparse
    g = sparse(n,1);
    for k = 1:nnz
      i = fscanf(fdat,'%i',1);
      g(i) = fscanf(fdat,'%e',1);
    end
  end

  % Hessian

  nnz = fscanf(fdat,'%i',1);			% type of Hessian
  fgets(fdat);					% next line to skip the comment
  if nnz < 0					% H is full
    H = fscanf(fdat,'%e',[n,n]);
  else						% H is sparse
    H = sparse(n,n);
    for k = 1:nnz
      i = fscanf(fdat,'%i',1);
      j = fscanf(fdat,'%i',1);
      H(i,j) = fscanf(fdat,'%e',1);
    end
  end

  % bounds on x

  lbx = -inf*ones(n,1);
  nlx = fscanf(fdat,'%i',1);			% nb of lower bounds on x
  fgets(fdat);					% next line to skip the comment
  for i=1:nlx
    j = fscanf(fdat,'%i',1);
    lbx(j) = fscanf(fdat,'%e',1);
  end
  ubx = inf*ones(n,1);
  nux = fscanf(fdat,'%i',1);			% nb of upper bounds on x
  fgets(fdat);					% next line to skip the comment
  for i=1:nux
    j = fscanf(fdat,'%i',1);
    ubx(j) = fscanf(fdat,'%e',1);
  end
  nb = sum(isfinite(lbx)|isfinite(ubx));	% nb of variables subject to a bound

  % inequality constraints

  mi = fscanf(fdat,'%i',1);
  fgets(fdat);					% next line to skip the comment

  if mi

    nnz = fscanf(fdat,'%i',1);			% type of Jacobian
    fgets(fdat);				% next line to skip the comment
    if nnz < 0					% AI is full
      AI = fscanf(fdat,'%e',[mi,n]);
    else					% AI is sparse
      AI = sparse(mi,n);
      for k = 1:nnz
        i = fscanf(fdat,'%i',1);
        j = fscanf(fdat,'%i',1);
        AI(i,j) = fscanf(fdat,'%e',1);
      end
    end

    lbi = -inf*ones(mi,1);
    nlbi = fscanf(fdat,'%i',1);			% nb of lower inequality constraints
    fgets(fdat);				% next line to skip the comment
    if nlbi
      for i=1:nlbi
        j = fscanf(fdat,'%i',1);
        lbi(j) = fscanf(fdat,'%e',1);
      end
      fgets(fdat);				% why a next line?
    end

    ubi = inf*ones(mi,1);
    nubi = fscanf(fdat,'%i',1);			% nb of upper inequality constraints
    fgets(fdat);				% next line to skip the comment
    if nubi
      for i=1:nubi
        j = fscanf(fdat,'%i',1);
        ubi(j) = fscanf(fdat,'%e',1);
      end
    end

  else

    AI = [];
    lbi = [];
    ubi = [];

  end

  % equality constraints

  me = fscanf(fdat,'%i',1);
  fgets(fdat);					% next line to skip the comment or to see whether the eof is reached
  mie = mi+me;

  if me
    nnz = fscanf(fdat,'%i',1);			% type of Jacobian
    fgets(fdat);				% next line to skip the comment
    if nnz < 0					% AE is full
      AE = fscanf(fdat,'%e',[me,n]);
    else					% AE is sparse
      AE = sparse(me,n);
      for k = 1:nnz
        i = fscanf(fdat,'%i',1);
        j = fscanf(fdat,'%i',1);
        AE(i,j) = fscanf(fdat,'%e',1);
      end
    end

    nnz = fscanf(fdat,'%i',1);			% type of rhs
    fgets(fdat);				% next line to skip the comment
    if nnz < 0					% be is full
      be = fscanf(fdat,'%e',me);
    else					% be is sparse
      be = sparse(me,1);
      for k = 1:nnz
        i = fscanf(fdat,'%i',1);
        be(i) = fscanf(fdat,'%e',1);
      end
    end
  else
    AE = [];
    be = [];
  end

  % force the matrice to be dence

  if force_full_matrices
    g  = full(g);
    H  = full(H);
    if mi
      AI = full(AI);
    end
    if me
      AE = full(AE);
      be = full(be);
    end
  end

  % solution

  if ~feof(fdat)

    nnz = fscanf(fdat,'%i',1);
    fgets(fdat);				% next line to skip the comment
    if nnz < 0					% the primal solution is full
      xopt = fscanf(fdat,'%e',n);
      xopt = xopt(:);
    else					% the primal solution is sparse
      xopt = sparse(n,1);
      for k = 1:nnz
        i = fscanf(fdat,'%i',1);
        xopt(i) = fscanf(fdat,'%e',1);
      end
    end

    nnz = fscanf(fdat,'%i',1);
    fgets(fdat);				% next line to skip the comment
    if nnz < 0					% the dual associated with the bounds is full
      lmopt = fscanf(fdat,'%e',n);
      lmopt = lmopt(:);
    else					% the dual solution associated with the bounds is sparse
      lmopt = sparse(n,1);
      for k = 1:nnz
        i = fscanf(fdat,'%i',1);
        lmopt(i) = fscanf(fdat,'%e',1);
      end
    end

    if mi
      nnz = fscanf(fdat,'%i',1);
      fgets(fdat);				% next line to skip the comment
      if nnz < 0				% the dual solution associated with the inequality constraints is full
        lmopti = fscanf(fdat,'%e',mi);
      else					% the dual solution associated with the inequality constraints is sparse
        lmopti = sparse(mi,1);
        for k = 1:nnz
          i = fscanf(fdat,'%i',1);
          lmopti(i) = fscanf(fdat,'%e',1);
        end
      end
      lmopt = [lmopt;lmopti];
    end

    if me
      nnz = fscanf(fdat,'%i',1);
      fgets(fdat);				% next line to skip the comment
      if nnz < 0				% the dual solution associated with the equality constraints is full
        lmopte = fscanf(fdat,'%e',me);
      else					% the dual solution associated with the equality constraints is sparse
        lmopte = sparse(me,1);
        for k = 1:nnz
          i = fscanf(fdat,'%i',1);
          lmopte(i) = fscanf(fdat,'%e',1);
        end
      end
      lmopt = [lmopt;lmopte];
    end

  end

  fclose(fdat);

  % Compute a perturbed solution

  if start_with_pert_lm

%   % perturbation in sup-norm

%   p = 1.e-2;	% size of the perturbation

%   xopt = xopt+(2*rand([n,1])-1)*p*norm(xopt,inf);				% perturbed xopt

%   if nb									% perturbed lmopt(1:n)
%     lmopt(1:n) = lmopt(1:n)+(2*rand([n,1])-1)*p*norm(lmopt(1:n),inf);
%   end
%   if mi									% perturbed lmopt(n+1:n+mi)
%     lmopt(n+1:n+mi) = lmopt(n+1:n+mi)+(2*rand([mi,1])-1)*p*norm(lmopt(n+1:n+mi),inf);
%   end
%   if me									% perturbed lmopt(n+mi+1:n+mi+me)
%     lmopt(n+mi+1:n+mi+me) = lmopt(n+mi+1:n+mi+me)+(2*rand([me,1])-1)*p*norm(lmopt(n+mi+1:n+mi+me),inf);
%   end
%   lmopt = lmopt(:);

    % perturbation in Euclidean norm

    p = 1.e-2;	% size of the perturbation

    % perturbed xopt
    pert = rand*p*norm(xopt)*ones(n,1);		% radius
    for i = 1:n-1
      c = 2*rand-1;
      pert(i) = pert(i)*c;			% *cos
      pert(i+1:n) = pert(i+1:n)*sin(acos(c));	% *sin
    end
    xopt = xopt+pert;

    % perturbed lmopt(1:n)
    if nb
      pert = rand*p*norm(lmopt(1:n))*ones(n,1);	% radius
      for i = 1:n-1
        c = 2*rand-1;
        pert(i) = pert(i)*c;			% *cos
        pert(i+1:n) = pert(i+1:n)*sin(acos(c));	% *sin
      end
      lmopt(1:n) = lmopt(1:n)+pert;
    end

    % perturbed lmopt(n+1:n+mi)
    if mi
      pert = rand*p*norm(lmopt(n+1:n+mi))*ones(mi,1);	% radius
      for i = 1:mi-1
        c = 2*rand-1;
        pert(i) = pert(i)*c;				% *cos
        pert(i+1:mi) = pert(i+1:mi)*sin(acos(c));	% *sin
      end
      lmopt(n+1:n+mi) = lmopt(n+1:n+mi)+pert;
    end

    % perturbed lmopt(n+mi+1:n+mi+me)
    if me
      pert = rand*p*norm(lmopt(n+mi+1:n+mi+me))*ones(me,1);	% radius
      for i = 1:me-1
        c = 2*rand-1;
        pert(i) = pert(i)*c;				% *cos
        pert(i+1:me) = pert(i+1:me)*sin(acos(c));	% *sin
      end
      lmopt(n+mi+1:n+mi+me) = lmopt(n+mi+1:n+mi+me)+pert;
    end

    lmopt = lmopt(:);

  end

%printdata (g,H,lbx,ubx,AI,lbi,ubi,AE,be)
%return
  % Print data

% x = zeros(n,1);
% fprintf('\n i      lbi(i)         x(i)         ubi(i)');
% for i=1:n
%   fprintf('\n%2i  %12.5e  %12.5e  %12.5e',i,lbx(i),x(i),ubx(i));
%   if x(i) <= lbx(i)+1.e-8; fprintf('  bound-'); end
%   if x(i) >= ubx(i)-1.e-8; fprintf('  bound+'); end
% end

  eigH = eig(H);
  eigHmin = min(eigH);
  eigHmax = max(eigH);
  fprintf(fid,'\nEigenvalues of H: min = %9.2e, max = %8.2e, cond = %8.2e',eigHmin,eigHmax,eigHmax/eigHmin);

  Hp = H;
  if me & ~strcmp(datafile,'cutest_aug3d.tdt')	% too long for 'cutest_aug3d.tdt'
    Hp = Hp+AE'*AE;
    eigHp = eig(Hp);
    eigHmin = min(eigHp);
    eigHmax = max(eigHp);
    fprintf(fid,'\nEigenvalues of H+AE''*AE: min = %9.2e, max = %8.2e, cond = %8.2e',eigHmin,eigHmax,eigHmax/eigHmin);
  end

  fprintf(fid,'\n');

%>  % Direct solver
%>
%>  fprintf('\nComputing by a direct solver\n');
%>
%>  time_init = clock;
%>  x0 = -H\g;
%>  fprintf(fid,'elapsed_time in a direct solve = %g sec',etime(clock,time_init));
%>
%>  % Direct solver
%>
%>  fprintf('\n\nComputing by a Cholesky factorization\n');
%>
%>  time_init = clock;
%>  [R,p] = chol(H);
%>  if p	% H is not positive definite
%>    return
%>  end
%>  x0 = -(R')\g;
%>  x0 = R\x0;
%>  fprintf(fid,'elapsed_time in a Cholesky factorization = %g sec',etime(clock,time_init));

%d=zeros(n,1);
%d(1)=1;
%d(2)=1;
%d=-d;
%AId=AI*d
%lbi
%ubi
%AEd=AE*d
%stop

%[flag] = printdata (g,H,lbx,ubx,AI,lbi,ubi,AE,be)

%----------------------
% Solve the QP by QPALM
%----------------------

  fprintf(fid,'\nRunning QPALM\n');

  if start_with_good_x & exist('xopt')
    x0  = xopt;
  else
    x0  = [];
  end
  if start_with_good_lm & exist('lmopt')
    lm0 = lmopt;
  else
    lm0 = [];
  end
% lm0 = zeros(2,1);	% for solved2b.tdt
% lm0(1) = -2/3;
% lm0(2) = -1/3;

  options.feas    = 1.e-8;
  options.feass   = 1.e-8;
% options.dxmin   = 1.e-12;
  options.gtol    = 1.e-8;
  options.ubtol   = 1.e-8;
  options.fout    = fid;
  options.verb    = 5;
  options.rlag    = 1.e+00;
% options.dcr     = 1.e-5;
  options.alit    = 30;
% options.phase   = 3;
% options.cput    = 0.2;
% options.gp_pls  = 'armijo';		% 'exact', 'armijo', 'goldstein'
% options.gp_init = 'curv-init'; 	% 'curv-init' (default), 'inc-bkpt'
% options.gp_succ = 'mt';		% 'one' (default), 'mt'
% options.cgph    = 30;
% options.cgit    = 5;
% options.cgprec  = 'none';		% 'none', 'diag', 'chol'
% options.cg_as   = 'more-toraldo';	% 'hit-and-fix' (default), 'more-toraldo'

% for getting the convergence rate
% lm0  = 1.e5*ones(mie,1);
% options.gtol    = 1.e-10;
% options.ubtol   = 1.e-10;
% options.alit    = 300;
  options.rctl    = 'constraint-change';	% 'fixed', 'constraint', 'constraint-change'
% options.rlag    = 3.0e+2;
% options.dcrf    = 0;	% 1.e-10;
% options.x0it    = 10;

  if strcmp(datafile,'cutest_qpcboe12.tdt')
    options.feas   = 1.e-5;
%   options.gtol   = 1.e-8;
%   options.dcr    = 5.e-1;
%   options.alit   = 200;
  end
  if strncmp(datafile,'gilbert-delbos-2005-ex43-',25)
    lm0(1) = 1.e2;
    options.gtol = 1.e-15;
    options.dcrf = 1.e-10;
  end

  time_init = clock;
  [x1,lm1,info] = qpalm(g,H,lbx,ubx,AI,lbi,ubi,AE,be,x0,lm0,options);
  fprintf(fid,'elapsed_time in QPALM = %g sec\n',etime(clock,time_init));

  switch info.flag
  case 0
  case 1
    fprintf(fid,'\nSmallest feasible shift s');
    fprintf(fid,'\n  %22.15e',info.s);
  case 2
    return
  case 10
    fprintf(fid,'\nDirection of unboundedness d');
    fprintf(fid,'\n  %12.5e',info.ubdd);
    fprintf(fid,'\nH*d =');
    fprintf(fid,'\n  %12.5e',H*info.ubdd);
% otherwise
%   return
  end

%g
%H
%gradL=g+H*x1;
%lmx = lm1(1:n)
%if nlx+nux; gradL = gradL+lm1(1:n); end
%if mi; gradL = gradL+lm1(n+1:n+mi); end
%if me; gradL = gradL+lm1(n+mi+1:n+mie); end
%gradL

  %-------------------------
  % solve the QP by QUADPROG
  %-------------------------

  fprintf(fid,'\n\nRunning QUADPROG\n');

% qp_options = optimset( ...
%      'LargeScale','off', ...
%      'Display','iter', ...
%      'TolFun',1e-8, ...
%      'TolX',1e-8);

  qp_options            = optimset();	% define the structure 'qp_options' with empty fields
  qp_options.LargeScale = 'off';	% otherwise Quadprog-active-set switches to the trust-region-reflective algorithm
  qp_options.Algorithm  = 'active-set';
% qp_options.Algorithm  = 'interior-point-convex';
  qp_options.Display    = 'iter';
  qp_options.TolFun     = 1.e-8;
  qp_options.TolX       = 1.e-8;
  qp_options.TolCon     = 1.e-8;
  qp_options.MaxIter    = 10000;

  if strcmp(datafile,'cutest_qpcboe12.tdt')
    qp_options.TolCon   = 1.e-5;
  end

  tic;
  if mi
    [x2,f,exitflag,output,lm2] = quadprog(H,g,[AI;-AI],[ubi;-lbi],AE,be,lbx,ubx,x0,qp_options);
  else
    [x2,f,exitflag,output,lm2] = quadprog(H,g,[],[],AE,be,lbx,ubx,x0,qp_options);
  end
  fprintf(fid,'Algorithm used: %s\n',output.algorithm);
  fprintf(fid,'Elapsed_time in QUADPROG = %g sec',toc);

  % Check optimality at the current x. It is checked whether the
  % projected gradient 'pgr' is small enough. Recall that the projected
  % gradient 'pgr' is defined by
  %
  %             { min(0,gr(i))  if lbx(i) = x(i)
  %    pgr(i) = { gr(i)         if lbx(i) < x(i) < ubx(i)
  %             { max(0,gr(i))  if          x(i) = ubx(i),
  %
  %  where gr is the gradient of the objective.

  pgr = g+H*x2;
  if mi; pgr = pgr+AI'*(lm2.ineqlin(1:mi)-lm2.ineqlin(mi+1:mi+mi)); end
  if me; pgr = pgr+AE'*lm2.eqlin; end
  Il = find(x2<=lbx+options.feas);	% indices of the variables at the lower bound
  pgr(Il) = min(0,pgr(Il));
  Iu = find(ubx-options.feas<=x2);	% indices of the variables at the upper bound
  pgr(Iu) = max(0,pgr(Iu));

  fprintf(fid,'\nFunction value: %12.5e',f);
  fprintf(fid,'\nActive bounds:\n. %i lower bounds',length(Il));
  if length(Il) > 0
    fprintf(fid,':');
    fprintf(fid,' %i',Il);
  end
  fprintf(fid,'\n. %i upper bounds',length(Iu));
  if length(Iu) > 0
    fprintf(fid,':');
    fprintf(fid,' %i',Iu);
  end

  fprintf(fid,'\n|Projected gradient| = %12.5e',norm(pgr,inf));

  %--------------------------
  % Compare the two solutions
  %--------------------------

  fprintf(fid,'\n\nDifference between the two algorithms');
  x1n = norm(x1);
  if x1n ~= 0
    fprintf(fid,'\n- primal (relative): %12.5e\n',norm(x2-x1)/x1n);
  else
    fprintf(fid,'\n- primal (absolute): %12.5e\n',norm(x2-x1));
  end

  fprintf(fid,'\nSolution');
  fprintf(fid,'\n^^^^^^^^');
    fprintf(fid,'\n                              Qpalm                       Quadprog');
    fprintf(fid,'\n                     --------------------------  --------------------------');
  fprintf(fid,'\nindex       lbx            x        multiplier         x        multiplier     ubx');
  dmax = 0;
  for i=1:n
    fprintf(fid,'\n%5i  %12.5e  %12.5e  %12.5e  %12.5e  %12.5e  %12.5e', ...
      i,lbx(i),x1(i),lm1(i),x2(i),lm2.upper(i)-lm2.lower(i),ubx(i));
    if abs(x2(i)-x1(i)) > dmax
      imax = i;
      dmax = abs(x2(i)-x1(i));
    end
  end
  fprintf(fid,'\nMax diff %12.5e',dmax);
  if dmax; fprintf(fid,', at index %i',imax); end
  fprintf(fid,'\n');

  if mi
    fprintf(fid,'\nInequality constraints');
    fprintf(fid,'\n^^^^^^^^^^^^^^^^^^^^^^');
    fprintf(fid,'\n                              Qpalm                       Quadprog');
    fprintf(fid,'\n                     --------------------------  --------------------------');
    fprintf(fid,'\nindex       lbi          AI*x       multiplier       AI*x       multiplier       ubi');
    AIx1 = AI*x1;
    AIx2 = AI*x2;
    dmax = 0;
    for i=1:mi
      fprintf(fid,'\n%5i  %12.5e  %12.5e  %12.5e  %12.5e  %12.5e  %12.5e', ...
        i,lbi(i),AIx1(i),lm1(n+i),AIx2(i),lm2.ineqlin(i)-lm2.ineqlin(mi+i),ubi(i));
      if abs(AIx2(i)-AIx1(i)) > dmax
        imax = i;
        dmax = abs(AIx2(i)-AIx1(i));
      end
    end
    fprintf(fid,'\nMax diff %12.5e',dmax);
    if dmax; fprintf(fid,', at index %i',imax); end
    fprintf(fid,'\n');
  end

  if me
    fprintf(fid,'\nEquality constraints');
    fprintf(fid,'\n^^^^^^^^^^^^^^^^^^^^');
    fprintf(fid,'\n                 Qpalm                      Quadprog');
    fprintf(fid,'\n       --------------------------  --------------------------');
    fprintf(fid,'\nindex     AE*x-b      multiplier      AE*x-b      multiplier');
    ce1 = AE*x1-be;
    ce2 = AE*x2-be;
    dmax = 0;
    for i=1:me
      fprintf(fid,'\n%5i  %12.5e  %12.5e  %12.5e  %12.5e',i,ce1(i),lm1(n+mi+i),ce2(i),lm2.eqlin(i));
      if abs(ce2(i)-ce1(i)) > dmax
        imax = i;
        dmax = abs(ce2(i)-ce1(i));
      end
    end
    fprintf(fid,'\nMax diff %12.5e',dmax);
    if dmax; fprintf(fid,', at index %i',imax); end
    fprintf(fid,'\n');
  end

  %--------------------------
  % Saving the solution
  %--------------------------

  fsol = fopen('sol.txt','w');
  if fsol < 0
    fprintf ('\n### Cannot open file ''%s'' for writing\n\n','sol.txt');
    return
  end

  fprintf(fsol,'%23.16e          primal solution\n',x1(1));
  if n > 1; fprintf(fsol,'%23.16e\n',x1(2:n)); end

  fprintf(fsol,'%23.16e          dual solution associated with the bound constraints\n',lm1(1));
  if n > 1; fprintf(fsol,'%23.16e\n',lm1(2:n)); end

  if mi
    fprintf(fsol,'%23.16e          dual solution associated with the inequality constraints\n',lm1(n+1));
    if mi > 1; fprintf(fsol,'%23.16e\n',lm1(n+2:n+mi)); end
  end

  if me
    fprintf(fsol,'%23.16e          dual solution associated with the equality constraints\n',lm1(n+mi+1));
    if me > 1; fprintf(fsol,'%23.16e\n',lm1(n+mi+2:n+mie)); end
  end

  return
